package com.chinacreator.ecd.flowstatus;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import com.chinacreator.ecd.changshipa.service.ChangshiPublicAccountService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.chinacreator.ecd.util.GenerateSequenceUtil;

@Service
public class FlowstatusService {
	
	private final static Logger logger = LoggerFactory.getLogger(FlowstatusService.class);
	
	@Value("${flowstatus.batchsize}")
	private String batchsize;
	
	@Value("${flowstatus.saveduration}")
	private String saveduration;
	
	@Autowired
	private FlowstatusMapper flowstatusMapper;

	@Autowired
	private ChangshiPublicAccountService changshiPublicAccountService;
	
	private long lastSaveTime = System.currentTimeMillis();
	private List<FlowstatusEntity> flows = new ArrayList<>();
	private Lock lock = new ReentrantLock();

	private static int i = 0;

	/**
	 * 
	 * @param dataString
	 */
	public void saveFlowuseData(String dataString) {
		this.lock.lock();
		FlowstatusEntity entity = JSON.parseObject(dataString, FlowstatusEntity.class);
		entity.setId(GenerateSequenceUtil.generateSequenceNo());
		flows.add(entity);
		
		if (flows.size() < Integer.valueOf(batchsize)) {
			logger.info("流量状态缓存了 {} 条，达到 {} 条或距上次保存超过 {} min后开始批量保存", flows.size(), batchsize, saveduration);
		}
		
		// 1000条批量插
		if (flows.size() >= Integer.valueOf(batchsize)) {

			logger.info("流量状态缓存达到 {} 条或距上次保存超过 {} min，开始批量保存", flows.size(), saveduration);
			
			saveFlowstatusData();
		}

		try {
			Map<String, String> param = new HashMap<>();
			param.put("usermob", entity.getSerialNumber());
			param.put("elementid", entity.getElementId());
			param.put("busiid", "flowout");
			param.put("usedflow", entity.getUsedValue() + "GB");
			param.put("balance", entity.getRemain() + "GB");


			if ("100".equals(entity.getPercent()) && i < 2) {
//				i++;// 测试
//				param.put("usermob", "17610768237"); // 测试

				changshiPublicAccountService.send(param);
			}


		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("调用畅视小程序模板消息接口失败");
		}

		this.lock.unlock();
	}
	
	/**
	 * 是否超过{saveduration}min没有保存过了
	 */
	public void flushFlowstatusData() {
		if ((System.currentTimeMillis() - lastSaveTime) >= 1000 * 60 * Integer.valueOf(saveduration)) {
			if (flows.size() == 0) {
				logger.info("检查流量状态缓存时长，距上次保存达到 {} min，目前无缓存", saveduration, flows.size());
				return;
			}
			logger.info("检查流量状态缓存时长，距上次保存达到 {} min，目前有缓存 {}，开始批量保存", saveduration, flows.size());
			
			saveFlowstatusData();
		} else {
			
			long duration = System.currentTimeMillis() - lastSaveTime;
			logger.info("检查流量状态缓存时长，距上次保存 {} ms = {} min，未达到 {} min, 缓存有 {} 条", duration, (duration/1000/60), saveduration, flows.size());
		
		}
	}
	
	private void saveFlowstatusData() {
		if (this.flows.size() != 0) {
            this.lock.lock();
		
		long duration = System.currentTimeMillis() - lastSaveTime;
		logger.info("流量状态缓存达到 {} 条，距上次保存 {} ms，开始批量保存", flows.size(), duration);
		
		try {
			
			long starttime = System.currentTimeMillis();
			flowstatusMapper.save(flows);
			
			long endedtime = System.currentTimeMillis();
			logger.info("流量状态保存成功，数量={}，用时={}ms,", flows.size(), endedtime - starttime);
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("流量状态保存失败，数量={}，失败原因={}, 失败数据={}", flows.size(), e.getMessage(), JSON.toJSONString(flows));
		}
		
		flows.clear();
		lastSaveTime = System.currentTimeMillis();
		this.lock.unlock();
	}
}
}
